@@ -6,8 +6,8 @@
 ;;         Abdelhak Bougouffa <abougouffa@fedoraproject.org>
 ;; Maintainer: Abdelhak Bougouffa <abougouffa@fedoraproject.org>
 ;; Created: 2013-03-02
-;; Package-Version: 1.2
-;; Package-Revision: 5b7f7b06ecff
+;; Package-Version: 20250828.2224
+;; Package-Revision: 0299c129d415
 ;; Package-Requires: ((emacs "27"))
 ;; Keywords: tools c
 ;; URL: https://github.com/jart/disaster
@@ -117,6 +117,11 @@
   :group 'disaster
   :type 'string)
 
+(defcustom disaster-meson-build-directory "build"
+  "Name of the Meson build directory."
+  :group 'disaster
+  :type 'string)
+
 (defcustom disaster-assembly-mode #'asm-mode
   "Which mode to use to view assembly code."
   :group 'disaster
@@ -257,6 +262,26 @@ PROJ-ROOT: path to project root, REL-FILE FILE."
                      (mapconcat (lambda (s) (concat "\"" s "\"")) argv " ")
                    (gethash "command" obj))))))))
 
+(defun disaster-create-compile-command-meson (make-root _cwd _rel-obj _obj-file _proj-root rel-file)
+  "Create compile command from Meson.
+MAKE-ROOT: path to build root,
+CWD: path to current source file,
+REL-OBJ: path to object file (relative to build root),
+OBJ-FILE: full path to object file (build root!)
+PROJ-ROOT: path to project root, REL-FILE FILE."
+  (let* ((json-object-type 'hash-table)
+         (json-array-type 'list)
+         (json-key-type 'string)
+         (json (json-read-file (expand-file-name "compile_commands.json" make-root))))
+    (catch 'compile-command
+      (dolist (obj json)
+        (let ((this-file (gethash "file" obj)))
+          (when (string-equal this-file rel-file)
+            (throw 'compile-command
+                   (if-let ((argv (gethash "arguments" obj)))
+                       (mapconcat (lambda (s) (concat "\"" s "\"")) argv " ")
+                     (gethash "command" obj)))))))))
+
 (defun disaster-get-object-file-path-compile-db (compile-cmd)
   "Get the .o object file name from a full COMPILE-CMD."
   (let* ((parts (reverse (split-string-and-unquote compile-cmd " ")))
@@ -266,19 +291,26 @@ PROJ-ROOT: path to project root, REL-FILE FILE."
         (when (member part '("-o" "--output")) (throw 'object-file prev))
         (setq prev part)))))
 
-(defun disaster-create-compile-command (use-compile-db make-root cwd rel-obj obj-file proj-root rel-file file)
+(defun disaster-create-compile-command (use-compile-db use-meson make-root cwd rel-obj obj-file proj-root rel-file file)
   "Create the actual compile command.
-- USE-COMPILE-DB: non NIL to use the compiler found in compile_commands.json,
-generated by CMake or Bear for example, NIL to use Make or default compiler
-options,
+- USE-COMPILE-DB: non NIL to use the compiler found in
+compile_commands.json,generated by CMake or Bear for example,
+NIL to use Make or default compiler options,
+- USE-MESON: non NIL to use the compiler found in
+Meson-generated compile_commands.json,
+NIL to use Make or default compiler options,
 - MAKE-ROOT: path to build root,
 - CWD: path to current source file,
 - REL-OBJ: path to object file (relative to project root),
 - OBJ-FILE: full path to object file (build root!)
 - PROJ-ROOT: path to project root, REL-FILE FILE."
-  (if use-compile-db
-      (disaster-create-compile-command-compile-db make-root cwd rel-obj obj-file proj-root rel-file)
-    (disaster-create-compile-command-make make-root cwd rel-obj obj-file proj-root rel-file file)))
+  (cond
+   (use-meson
+    (disaster-create-compile-command-meson make-root cwd rel-obj obj-file proj-root rel-file))
+   (use-compile-db
+    (disaster-create-compile-command-compile-db make-root cwd rel-obj obj-file proj-root rel-file))
+   (t
+    (disaster-create-compile-command-make make-root cwd rel-obj obj-file proj-root rel-file file))))
 
 ;;;###autoload
 (defun disaster (&optional file line)
@@ -312,19 +344,25 @@ is used."
         (let* ((cwd (file-name-directory file)) ;; path to current source file
                (proj-root (disaster-find-project-root nil file)) ;; path to project root
                (use-compile-db (file-exists-p (expand-file-name "compile_commands.json" proj-root)))
-               (make-root (disaster-find-build-root use-compile-db proj-root)) ;; path to build root
-               (default-directory (if use-compile-db proj-root default-directory))
-               (rel-file (if proj-root ;; path to source file (relative to project root)
-                             (file-relative-name file proj-root)
-                           file))
+               (use-meson (file-exists-p (expand-file-name "meson.build" proj-root)))
+               (make-root (disaster-find-build-root use-compile-db use-meson proj-root)) ;; path to build root
+               (default-directory (cond
+                                   (use-meson make-root)
+                                   (use-compile-db proj-root)
+                                   (t default-directory)))
+               (rel-file (cond
+                          ;; path to source file (relative to project root)
+                          (use-meson (file-relative-name file make-root))
+                          (proj-root (file-relative-name file proj-root))
+                          (t file)))
                (rel-obj (concat (file-name-sans-extension rel-file) ".o")) ;; path to object file (relative to project root)
                (obj-file (expand-file-name rel-obj make-root)) ;; full path to object file (build root!)
-               (cc (disaster-create-compile-command use-compile-db make-root cwd rel-obj obj-file proj-root rel-file file))
+               (cc (disaster-create-compile-command use-compile-db use-meson make-root cwd rel-obj obj-file proj-root rel-file file))
                (dump (format "%s %s" disaster-objdump (shell-quote-argument obj-file)))
                (line-text (buffer-substring-no-properties (line-beginning-position) (line-end-position))))
 
           ;; For CMake, read the object file from compile_commands.json
-          (when use-compile-db
+          (when (or use-compile-db use-meson)
             (let ((tmp (disaster-get-object-file-path-compile-db cc)))
               (when (file-exists-p (expand-file-name "CMakeLists.txt" proj-root))
                 (setq cc (format "cmake --build %s --target %s" make-root tmp)))
@@ -450,17 +488,21 @@ convenience. If LOOKS is not specified, it'll default to
       (setq looks (cdr looks)))
     res))
 
-(defun disaster-find-build-root (use-compile-db project-root)
+(defun disaster-find-build-root (use-compile-db use-meson project-root)
   "Find the root of build directory.
 USE-COMPILE-DB: non nil to use compile_commands.json,
 PROJECT-ROOT: root directory of the project."
-  (if use-compile-db
-      (progn
-        (let* ((json-object-type 'hash-table)
-               (json-array-type 'list)
-               (json-key-type 'string)
-               (json (json-read-file (expand-file-name "compile_commands.json" project-root))))
-          (file-name-as-directory (gethash "directory" (car json)))))
+  (cond
+   (use-meson
+    (concat project-root disaster-meson-build-directory "/"))
+   (use-compile-db
+    (progn
+      (let* ((json-object-type 'hash-table)
+             (json-array-type 'list)
+             (json-key-type 'string)
+             (json (json-read-file (expand-file-name "compile_commands.json" project-root))))
+        (file-name-as-directory (gethash "directory" (car json))))))
+   (t
     (and project-root
          (or (let (build-root
                    (funcs disaster-find-build-root-functions))
@@ -469,7 +511,7 @@ PROJECT-ROOT: root directory of the project."
                        funcs (cdr funcs)))
                (and build-root
                     (file-name-as-directory build-root)))
-             project-root))))
+             project-root)))))
 
 (provide 'disaster)
 
